@@ -38,7 +38,8 @@ in their names.
 **Available widgets:**
 
 :py:class:`.Select2Widget`, :py:class:`.Select2MultipleWidget`, :py:class:`.HeavySelect2Widget`, :py:class:`.HeavySelect2MultipleWidget`,
-:py:class:`.AutoHeavySelect2Widget`, :py:class:`.AutoHeavySelect2MultipleWidget`
+:py:class:`.AutoHeavySelect2Widget`, :py:class:`.AutoHeavySelect2MultipleWidget`, :py:class:`.HeavySelect2TagWidget`,
+:py:class:`.AutoHeavySelect2TagWidget`
 
 `Read more`_
 
@@ -57,7 +58,8 @@ your ease.
 :py:class:`.HeavySelect2MultipleChoiceField`, :py:class:`.HeavyModelSelect2ChoiceField`,
 :py:class:`.HeavyModelSelect2MultipleChoiceField`, :py:class:`.ModelSelect2Field`, :py:class:`.ModelSelect2MultipleField`,
 :py:class:`.AutoSelect2Field`, :py:class:`.AutoSelect2MultipleField`, :py:class:`.AutoModelSelect2Field`,
-:py:class:`.AutoModelSelect2MultipleField`
+:py:class:`.AutoModelSelect2MultipleField`, :py:class:`.HeavySelect2TagField`, :py:class:`.AutoSelect2TagField`,
+:py:class:`.HeavyModelSelect2TagField`, :py:class:`.AutoModelSelect2TagField`
 
 Views
 -----
@@ -77,7 +79,7 @@ The view - `Select2View`, exposed here is meant to be used with 'Heavy' fields a
 import logging
 logger = logging.getLogger(__name__)
 
-__version__ = "4.0.0"
+__version__ = "4.2.0"
 
 __RENDER_SELECT2_STATICS = False
 __ENABLE_MULTI_PROCESS_SUPPORT = False
@@ -89,6 +91,8 @@ __SECRET_SALT = ''
 
 try:
     from django.conf import settings
+    if logger.isEnabledFor(logging.INFO):
+        logger.info("Django found.")
     if settings.configured:
         __RENDER_SELECT2_STATICS = getattr(settings, 'AUTO_RENDER_SELECT2_STATICS', True)
         __ENABLE_MULTI_PROCESS_SUPPORT = getattr(settings, 'ENABLE_SELECT2_MULTI_PROCESS_SUPPORT', False)
@@ -103,12 +107,16 @@ try:
             __ENABLE_MULTI_PROCESS_SUPPORT = False
 
         from .widgets import Select2Widget, Select2MultipleWidget, HeavySelect2Widget, HeavySelect2MultipleWidget, \
-            AutoHeavySelect2Widget, AutoHeavySelect2MultipleWidget
+            AutoHeavySelect2Widget, AutoHeavySelect2MultipleWidget, HeavySelect2TagWidget, AutoHeavySelect2TagWidget
         from .fields import Select2ChoiceField, Select2MultipleChoiceField, HeavySelect2ChoiceField, \
             HeavySelect2MultipleChoiceField, HeavyModelSelect2ChoiceField, HeavyModelSelect2MultipleChoiceField, \
             ModelSelect2Field, ModelSelect2MultipleField, AutoSelect2Field, AutoSelect2MultipleField, \
-            AutoModelSelect2Field, AutoModelSelect2MultipleField 
+            AutoModelSelect2Field, AutoModelSelect2MultipleField, HeavySelect2TagField, AutoSelect2TagField, \
+            HeavyModelSelect2TagField, AutoModelSelect2TagField
         from .views import Select2View, NO_ERR_RESP
+
+        if logger.isEnabledFor(logging.INFO):
+            logger.info("Django found and fields and widgest loaded.")
 except ImportError:
     if logger.isEnabledFor(logging.INFO):
         logger.info("Django not found.")
