@@ -293,13 +293,23 @@ class MultipleSelect2HiddenInput(forms.TextInput):
                     % (id_, convert_to_js_arr(value, id_))
             jscode += u"django_select2.initMultipleHidden($('#%s'));" % id_
             s += render_js_script(jscode)
-        return s
+        return mark_safe(s)
 
     def value_from_datadict(self, data, files, name):
         if isinstance(data, (MultiValueDict, MergeDict)):
             return data.getlist(name)
         return data.get(name, None)
 
+    def _has_changed(self, initial, data):
+        if initial is None:
+            initial = []
+        if data is None:
+            data = []
+        if len(initial) != len(data):
+            return True
+        initial_set = set([force_unicode(value) for value in initial])
+        data_set = set([force_unicode(value) for value in data])
+        return data_set != initial_set
 
 ### Heavy mixins and widgets ###
 
@@ -356,7 +366,7 @@ class HeavySelect2Mixin(Select2Mixin):
 
                 3. Otherwise, check the cached results. When the user searches in the fields then all the returned
                 responses from server, which has the value and label mapping, are cached by ``heavy_data.js``.
-                
+
         :type userGetValTextFuncName: :py:obj:`str`
 
         .. tip:: Since version 3.2.0, cookies or localStorage are no longer checked or used. All
@@ -494,7 +504,7 @@ class HeavySelect2MultipleWidget(HeavySelect2Mixin, MultipleSelect2HiddenInput):
 
     Following Select2 options from :py:attr:`.Select2Mixin.options` are added or set:-
 
-        * multiple: ``False``
+        * multiple: ``True``
         * separator: ``JSVar('django_select2.MULTISEPARATOR')``
 
     """
@@ -529,6 +539,34 @@ class HeavySelect2MultipleWidget(HeavySelect2Mixin, MultipleSelect2HiddenInput):
             if texts:
                 return u"$('#%s').txt(%s);" % (id_, texts)
 
+class HeavySelect2TagWidget(HeavySelect2MultipleWidget):
+    """
+    Heavy widget with tagging support. Based on :py:class:`HeavySelect2MultipleWidget`,
+    unlike other widgets this allows users to create new options (tags).
+
+    Following Select2 options from :py:attr:`.Select2Mixin.options` are removed:-
+
+        * allowClear
+        * minimumResultsForSearch
+        * closeOnSelect
+
+    Following Select2 options from :py:attr:`.Select2Mixin.options` are added or set:-
+
+        * multiple: ``True``
+        * separator: ``JSVar('django_select2.MULTISEPARATOR')``
+        * tags: ``True``
+        * tokenSeparators: ``,`` and `` ``
+        * createSearchChoice: ``JSFunctionInContext('django_select2.createSearchChoice')``
+        * minimumInputLength: ``1``
+
+    """
+    def init_options(self):
+        super(HeavySelect2TagWidget, self).init_options()
+        self.options.pop('closeOnSelect', None)
+        self.options['minimumInputLength'] = 1
+        self.options['tags'] = True
+        self.options['tokenSeparators'] = [",", " "]
+        self.options['createSearchChoice'] = JSFunctionInContext('django_select2.createSearchChoice')
 
 ### Auto Heavy widgets ###
 
@@ -561,3 +599,7 @@ class AutoHeavySelect2Widget(AutoHeavySelect2Mixin, HeavySelect2Widget):
 class AutoHeavySelect2MultipleWidget(AutoHeavySelect2Mixin, HeavySelect2MultipleWidget):
     "Auto version of :py:class:`.HeavySelect2MultipleWidget`"
     pass
+
+class AutoHeavySelect2TagWidget(AutoHeavySelect2Mixin, HeavySelect2TagWidget):
+    "Auto version of :py:class:`.HeavySelect2TagWidget`"
+    pass
